==> qvwm_1.1.7.orig.tar.gz: qvwm-1.1.7.orig/src/extra_image.cc <==
/*
 * extra_image.cc
 *
 * Copyright (C) 1995-2000 Kenichi Kourai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with qvwm; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef USE_IMLIB

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Imlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "main.h"
#include "extra_image.h"

ImlibData* ExtraImage::m_idImlib;

ExtraImage::ExtraImage(char* filename)
{
  m_error = LoadImage(filename);
}

// create a temporary file because Imlib does not provide any functions
// to create an image from raw data...
ExtraImage::ExtraImage(char* data, int len)
{
  char* filename = tmpnam(NULL);
  FILE* fp;
  int fd;

  if (filename == NULL) {
    QvwmError("cannot create an unique temporary file name");
    m_error = -1;
    return;
  }

  // using tmpnam induces a race condition, which we can't avoid - but we do
  // our best to protect ourselves
  if ((fd = open(filename, O_RDWR | O_CREAT | O_EXCL,
		 S_IRUSR | S_IWUSR)) == -1 ||
      (fp = fdopen(fd, "w")) == NULL) {
    QvwmError("cannot create a temporary file: %s", filename);
    m_error = -1;
    return;
  }

  if (fwrite(data, len, 1, fp) < 1) {
    QvwmError("cannot write to a temporary file: %s", filename);
    m_error = -1;
    fclose(fp);
    return;
  }
    
  fflush(fp);
  m_error = LoadImage(filename);

  // *must* unlink before close
  unlink(filename);
  fclose(fp);
}

int ExtraImage::LoadImage(char* filename)
{
  m_im = Imlib_load_image(m_idImlib, filename);
  if (m_im == NULL)
    return -1;
    
  m_size = Dim(m_im->rgb_width, m_im->rgb_height);
  
  Imlib_render(m_idImlib, m_im, m_size.width, m_size.height);

  m_pix = Imlib_move_image(m_idImlib, m_im);
  m_mask = Imlib_move_mask(m_idImlib, m_im);

  XGCValues gcv;
  gcv.clip_mask = m_mask;
  m_gc = XCreateGC(display, root, GCClipMask, &gcv);

  m_import = True;  // pixmap and mask are managed by Imlib

  return 0;
}

// ~QvImage is not executed because m_import is True
ExtraImage::~ExtraImage()
{
  Imlib_destroy_image(m_idImlib, m_im);
  Imlib_free_pixmap(m_idImlib, m_pix);

  if (m_gc)
    XFreeGC(display, m_gc);
}

void ExtraImage::Initialize()
{
  m_idImlib = Imlib_init(display);
}

#endif // USE_IMLIB

==> qvwm-1.1.8.tar.gz: qvwm-1.1.8/src/extra_image.cc <==
/*
 * extra_image.cc
 *
 * Copyright (C) 1995-2000 Kenichi Kourai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with qvwm; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef USE_IMLIB

#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Imlib.h>
#include "main.h"
#include "extra_image.h"

ImlibData* ExtraImage::m_idImlib;

ExtraImage::ExtraImage(char* filename)
{
  m_error = LoadImage(filename);
}

// create a temporary file because Imlib does not provide any functions
// to create an image from raw data...
ExtraImage::ExtraImage(char* data, int len)
{
  char filename[16] = "/tmp/qvwmXXXXXX";  /* must not be constant */
  int fd;

  if ((fd = mkstemp(filename)) < 0) {
    QvwmError("cannot create a temporary file");
    m_error = -1;
    return;
  }

  if (write(fd, data, len) < len) {
    QvwmError("cannot write to a temporary file");
    m_error = -1;
    close(fd);
    return;
  }
    
  m_error = LoadImage(filename);

  unlink(filename);
  close(fd);
}

int ExtraImage::LoadImage(char* filename)
{
  m_im = Imlib_load_image(m_idImlib, filename);
  if (m_im == NULL)
    return -1;
    
  m_size = Dim(m_im->rgb_width, m_im->rgb_height);
  
  Imlib_render(m_idImlib, m_im, m_size.width, m_size.height);

  m_pix = Imlib_move_image(m_idImlib, m_im);
  m_mask = Imlib_move_mask(m_idImlib, m_im);

  XGCValues gcv;
  gcv.clip_mask = m_mask;
  m_gc = XCreateGC(display, root, GCClipMask, &gcv);

  m_import = True;  // pixmap and mask are managed by Imlib

  return 0;
}

// ~QvImage is not executed because m_import is True
ExtraImage::~ExtraImage()
{
  Imlib_destroy_image(m_idImlib, m_im);
  Imlib_free_pixmap(m_idImlib, m_pix);

  if (m_gc)
    XFreeGC(display, m_gc);
}

void ExtraImage::Initialize()
{
  m_idImlib = Imlib_init(display);
}

#endif // USE_IMLIB
